#!/usr/bin/php
<?php

    // Set error reporting level
    error_reporting(E_ALL & ~E_NOTICE & ~E_WARNING);
    
    // Set working directory
    chdir('/home/alvaro');

    // Welcome message
    echo "\n####################################################\n\n";
    echo "####### MÀQUINA DE RECORDS by Àlvaro Cànovas #######\n\n";
    echo "####################################################\n\n";
    sleep(1);
    echo "> Stop script with SIGINT [Ctrl+C] \n\n";
    echo "> Starting... dir = " . getcwd() . "\n\n\n";
    echo "____________________________________________________\n\n\n";
    
    // PDF generator tools
    require('aux/TCPDF/tcpdf.php');
    // $tcpdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
    
    // Get next image id iterator 
    $i = get_next_record_iterator();
    while(TRUE) 
    {
		// Select working mode randomly [5%]
		$mode = rand(0,19);

		if($mode >= -9999) // Capture 'record' working mode
		{ 	
			// Generate filename
			$filename = "record_" . $i . ".jpg";
			echo "Click!\n\n";
			// Capture image
			shell_exec("raspistill -n -t 1 -w 2160 -h 3439 -o " . $filename);  
			$filesize = number_format(((filesize($filename) / 1024) / 1024), 2, ".", "");
			echo "Capturing '$filename' [" . $filesize . "MB]\n\n";
			if(!file_exists($filename)) { break; } // Errors found 
			
			// Generate pdf document in this directory
			$docname = "doc_" . $i . ".pdf";
			$tcpdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, "A6", true, 'UTF-8', false); // initialize TCPDF
			$pdfcontent = generate_document($tcpdf, $i, $filename, $filesize);
			if(!file_put_contents($docname, $pdfcontent, LOCK_EX)) { break; } // Errors found
			
			// Remove captured image once the document is created
			if(!unlink($filename)) { break; } // Errors found
			
			$docsize = number_format(((filesize($docname) / 1024) / 1024), 2, ".", "");
			echo "Generating document '$docname' [" . $docsize . "MB]\n\n";

			$dirsize = shell_exec("du -s HDD/ | cut -f 1");
			$freespace = disk_free_space("HDD/");
			
			echo "HDD free space: " . number_format((($freespace / 1024) / 1024), 2, ".", "") . " MB\n\n";
			
			if($freespace < 536870912) { // 512 MB
				// print 'record' document via CUPS
				$printer = "[WIFI]_Brother_DCP-1610W"; // set printer
				echo "Printing captured image via " . $printer . "\n\n";
				shell_exec("lp -d " . $printer . " -o media=A6 -o fit-to-page " . $docname);
				// shell_exec("lp " . $docname);
				
				// specify destination folder
				$outdir = "printed/";    
			}
			else {
				// move 'record' document to HDD
				echo "Moving captured image to the Hard Drive\n\n";
				// specify destination folder
				$outdir = "HDD/";
			}
				
			// move document to specified folder [HDD/ or printed/]
			if(!rename($docname,  $outdir . $docname))  { break; } // Errors found 
			
			// Increment iterator
			$i += 1;
		}
		else // Retrieve 'record' working mode
		{
			echo "Retrieving 'record'\n\n";
			echo "Place the document inside the scanner\n\n";
			$wait = 12; // seconds
			echo str_repeat("-", $wait) . "\n";
			for($j = 0; $j < $wait; $j++) { echo "#"; sleep(1); }
			echo "\n\n";
			
			// Start scanning
			echo "Scanning...\n\n";
			shell_exec("scanimage --format=tiff | convert tiff:- scan.pdf");
			
			$replace_id = rand(1, get_last_HDD_iterator()); // Get 'record' id that will be replaced
			// Move scanned 'record' document to the HDD replacing an old 'record'
			echo "Replacing document containing 'record " . $replace_id . "' with the scanned one\n\n";
			if(!rename("scan.pdf", "HDD/doc_" . $replace_id . ".pdf")) { break; } // Errors found		
		}
    
		// Sleep
		$time = rand(0,30); // seconds
		echo "I am going to sleep for " . $time . " seconds.\n\n";
		echo str_repeat("-", $time) . "\n"; 
		for($t = 0; $t < $time; $t++) {
            echo "#";
            sleep(1); // 1 second
		}
		if($time != 0) { echo "\n\n"; }
		echo "____________________________________________________\n\n\n";
    }
    
    // Some error caused the loop to break
    echo "\n\n\n> ERROR!\n\n\n";

?>

<?php
    function get_next_record_iterator() 
    {
        $lastfileHDD     = shell_exec("ls -t HDD/ | grep 'doc_' | head -n 1");
        $lastfilePrinted = shell_exec("ls -t printed/ | grep 'doc_' | head -n 1");
        $iteratorHDD = str_replace(".pdf", "", substr($lastfileHDD, 4));
        $iteratorPrinted = str_replace(".pdf", "", substr($lastfilePrinted, 4));
		       
        $iterator = ($iteratorHDD > $iteratorPrinted) ? $iteratorHDD : $iteratorPrinted;
        
        return $iterator + 1; 
    }
?>

<?php
    function get_last_HDD_iterator() 
    {
        $lastfile = shell_exec("ls -t HDD/ | grep 'doc_' | head -n 1");
        $iterator = str_replace(".pdf", "", substr($lastfile, 4));
        
        return $iterator; 
    }
?>

<?php
    function generate_document($tcpdf, $i, $filename, $filesize)
    {
        // var $tcpdf is innstantiated previously
    
        $date = date('d-m-Y/ H:i:s/ ');

        $htmlcontent = "
<!DOCTYPE html>
<html>
    <head>
    </head>
    <body style=\"text-align: right;\">
		<img src=\"" . $filename . "\" style=\"width: 300px; float: right;\" alt=\"image\">
		<span style=\"display: block; font-size: 15px; color: #494949; padding-top: 485px;\">
			Record_" . $i . ".jpg
			<span style=\"font-size: 7px; color: #666666;\"> " . $date . $filesize*1024 . "KB/ 41.399469, 2.190975</span>
		</span>
    </body>
</html>\n";

        // set default monospaced font
        $tcpdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

        // set title of pdf
        $tcpdf->SetTitle('Maquina de Records - Record ' . $i);
        $tcpdf->SetAuthor('Alvaro Canovas');
        
        // set margins
        $tcpdf->SetMargins(16, 4, 4, true);
        $tcpdf->SetHeaderMargin(PDF_MARGIN_HEADER);
        $tcpdf->SetFooterMargin(PDF_MARGIN_FOOTER);

        // set header and footer in pdf
        $tcpdf->setPrintHeader(false);
        $tcpdf->setPrintFooter(false);
        $tcpdf->setListIndentWidth(3);

        // set auto page breaks
        $tcpdf->SetAutoPageBreak(false, 0);

        // set image scale factor
        $tcpdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

        $tcpdf->AddPage('P', 'A6');

        $tcpdf->SetFont('helvetica', '', 13);

        $tcpdf->writeHTML($htmlcontent, true, false, false, false, '');

        //Close and output PDF document
        $doc = $tcpdf->Output('demo.pdf', 'S');
		
        return $doc;
        
    }
	
?>

